 //  ----------------------------------------------------------------------------
 //          ATMEL Microcontroller Software Support  -  ROUSSET  -
 //  ----------------------------------------------------------------------------
 //  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 //  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 //  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 //  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 //  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 //  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 //  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 //  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 //  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 //  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //  ----------------------------------------------------------------------------
//*----------------------------------------------------------------------------
//* File Name           : main.c
//* Object              : main application written in C
//* Creation            : GGi   06/2006
//*
//*----------------------------------------------------------------------------
#include <stdio.h>
#include "AT91SAM9261.h"
#include "lib_AT91SAM9261.h"
#include "mmu.h"

extern void AT91F_DBGU_Printk(char *);

//*----------------------------------------------------------------------------
//* \fn    AT91F_Bench
//*----------------------------------------------------------------------------
void AT91F_Bench(void)
{
	unsigned int start, stop, i;//, dummy;
	int * sdram_to, * sdram_from; 
	AT91PS_PITC pPit = AT91C_BASE_PITC;
	char msg[64];

/* single beat */
		sdram_from = (int *)0x20000000;
		sdram_to = (int *)0x22000000;
	
		// Synchronize on 1ms period
		AT91F_PITSetPIV(pPit,99300000/16000);
		pPit->PITC_PIMR |= AT91C_PITC_PITEN;	 
		start = AT91F_PITGetPIVR(pPit)>>20;

		//single-beat access
		for(i=0; i<0x800000; i++) *sdram_to++ = *sdram_from++;

		stop = AT91F_PITGetPIVR(pPit)>>20;
		pPit->PITC_PIMR &= ~AT91C_PITC_PITEN;	 

		sprintf(msg, "single access : 64 Mbytes transferred in %d ms\n\r", stop-start);
		AT91F_DBGU_Printk(msg);

/* 4-word burst */

		{__asm("mov	r0,#0x20000000");
		__asm("mov	r1,#0x22000000");
		}

		// Synchronize on 1ms period
		AT91F_PITSetPIV(pPit,99300000/16000);
		pPit->PITC_PIMR |= AT91C_PITC_PITEN;	 
		start = AT91F_PITGetPIVR(pPit)>>20;

		//bust of 4
		for(i=0; i<0x200000; i++){
			// r0 = sdram_from
			// r1 = sdram_to
			__asm("ldmia	r0!,{r4-r7}");
			__asm("stmia	r1!,{r4-r7}");
		}
		stop = AT91F_PITGetPIVR(pPit)>>20;
		pPit->PITC_PIMR &= ~AT91C_PITC_PITEN;	 

		sprintf(msg, "4 word burst : 64 Mbytes transferred in %d ms\n\r", stop-start);
		AT91F_DBGU_Printk(msg);
}
//*----------------------------------------------------------------------------
//* \fn    AT91F_Bench8
//* \brief separated from AT91F_Bench because not enough registers are available
//*----------------------------------------------------------------------------
void AT91F_Bench8(void)
{
	unsigned int start, stop, i;//, dummy;
	int * sdram_to, * sdram_from; 
	AT91PS_PITC pPit = AT91C_BASE_PITC;
	char msg[64];
		{__asm("mov	r0,#0x20000000");
		__asm("mov	r1,#0x22000000");
		}

		// Synchronize on 1ms period
		AT91F_PITSetPIV(pPit,99300000/16000);
		pPit->PITC_PIMR |= AT91C_PITC_PITEN;	 
		start = AT91F_PITGetPIVR(pPit)>>20;
		
		//bust of 8
		for(i=0; i<0x100000; i++){
			__asm("ldmia	r0!,{r2-r9}");
			__asm("stmia	r1!,{r2-r9}");
		}
		stop = AT91F_PITGetPIVR(pPit)>>20;
		pPit->PITC_PIMR &= ~AT91C_PITC_PITEN;	 

		sprintf(msg, "8 word burst : 64 Mbytes transferred in %d ms\n\r", stop-start);
		AT91F_DBGU_Printk(msg);
}


//*----------------------------------------------------------------------------
//* \fn    AT91F_InitMMU
//* \brief Initialize MMU
//*----------------------------------------------------------------------------
void AT91F_InitMMU(unsigned int *pTranslationTable)
{
	int i;
	// Program the TTB
	AT91F_ARM_WriteTTB((unsigned int) pTranslationTable);
	// Program the domain access register
	AT91F_ARM_WriteDomain(0xC0000000); // domain 15: access are not checked

	// Reset table entries
	for (i = 0; i < 4096; ++i)
		pTranslationTable[i] = 0;
	// Program level 1 page table entry

	pTranslationTable[0x3] =
		(0x3 << 20) |  // Physical Address
		(1 << 10) |      // Access in supervisor mode
		(15 << 5) |      // Domain
		(1 << 4) |
		(1 << 3) |       // Cachable
		0x2;             // Set as 1 Mbyte section

	/* SDRAM entire mapping */
	for(i=0;i<64;i++) 
		pTranslationTable[0x200+i] =
		((0x200+i) << 20) |  // Physical Address
		(1 << 10) |      // Access in supervisor mode
		(15 << 5) |      // Domain
		(1 << 4) |
		(1 << 3) |       // Cachable
		0x2;             // Set as 1 Mbyte section

	pTranslationTable[0xFFF] =
		(0xFFF << 20) |  // Physical Address
		(1 << 10) |      // Access in supervisor mode
		(15 << 5) |      // Domain
		(1 << 4) |
		0x2;             // Set as 1 Mbyte section

	// Enable the MMU
	AT91F_EnableMMU();
}


//*----------------------------------------------------------------------------
//* \fn    main
//* \brief the main code
//*----------------------------------------------------------------------------

int main()
{

/* Master 0&1 = ARM9, Slave 5 = EBI */
/*EBI as default master for ARM*/
*AT91C_MATRIX_SCFG3 = AT91C_MATRIX_FIXED_DEFMSTR3_ARM926I|AT91C_MATRIX_FIXED_DEFMSTR3_ARM926D|
						AT91C_MATRIX_DEFMSTR_TYPE_FIXED_DEFMSTR;


	AT91F_DBGU_Printk("\n\r-I- WO MMU & D Cache Disabled\n\r");
	AT91F_DBGU_Printk("\n\r-I- I Cache Enabled to minimize code execution time\n\r");
	AT91F_EnableICache();

	AT91F_Bench();
	AT91F_Bench8();

	//the TTB is in a cachable area
	AT91F_InitMMU((unsigned int *) 0x310000);

	AT91F_DBGU_Printk("\n\r-I- MMU & I+D Cache Enabled\n\r");
	AT91F_EnableICache();
	AT91F_EnableDCache();

	AT91F_Bench();
	AT91F_Bench8();

	while (1);
}